unit MruMenu;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus;

type
   TSelFileEvent = procedure(Sender: TObject; FileName: string) of object;

  TMruMenu = class(TComponent)
  private
    { Private declarations }
    FOnSelFile : TSelFileEvent;        // This event fires when the user selects an Mru file on the menu
    FMainMenu  : TMainMenu;            // The Parent Menu used to append the Mru file list
    FParGroup  : byte;                 // The GroupIndex of the Main Menu Item to append to (usually File = 0)
    FMruGroup  : byte;                 // Unique Group Num given to Mru Menu Items. Needed to delete ONLY the Mru Menu items. 
    FNumFiles  : integer;              // Max number of Mru files (default is 4)
    FMruFile   : string;               // The file used to store the Mru files
    FShowPath  : boolean;              // Determines if file Path is displayed in the menu
    Files      : TStringList;          // Holds the Mru files
    procedure  OnMenuClick(Sender: TObject);
    procedure  Loaded; override;
    procedure  UpdateMenu;
    function   MruFilePath: string;
    procedure  Notification(AComponent: TComponent; Operation: TOperation); override;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure  AddFile(fn : string);
    procedure  ConnectToMenu;          // So the user can call it after changing all the properties
    procedure  Clear;                  // Resets all Mru file list to empty
  published
    { Published declarations }
    property OnSelectFile  : TSelFileEvent read FOnSelFile write FOnSelFile;
    property ParentMenu    : TMainMenu read FMainMenu write FMainMenu;
    property ParentGroup   : byte read FParGroup write FParGroup;
    property MruGroup      : byte read FMruGroup write FMruGroup;
    property NumFiles      : integer read FNumFiles write FNumFiles;
    property MruFile       : string read FMruFile write FMruFile;
    property ShowPath      : boolean read FShowPath write FShowPath;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TMruMenu]);
end;

constructor TMruMenu.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FParGroup := 0;                     // Default for "File" menu
   FMruGroup := 253;                   // This should be unique!
   FNumFiles := 4;
   FMruFile  := 'Mru.ini';
   FShowPath := False;
   Files     := TStringList.Create;
   FMainMenu := nil;
end;

destructor TMruMenu.Destroy;
begin
   // Save the Mru file list to the Mru file
   Files.SaveToFile(MruFilePath);

   // Unhook from the Main Menu
   // This is for the weird case where we get destroyed but the menu doesn't.
   // It would be dangerous because the Menu OnClick events would be invalid.
   // This section used to crash until I added a Notification handler.
   Files.Clear;
   UpdateMenu;

   Files.Free;

   inherited Destroy;
end;

procedure TMruMenu.AddFile(fn : string);
var
   i : integer;
begin
   // Delete file "fn" from Files (if its in there)
   i := Files.IndexOf(fn);
   if i<>-1 then Files.Delete(i);

   // Append "fn" to the top of Files (since it is the Most Recently Used)
   Files.Insert(0, fn);

   // Trim the oldest files if more than NumFiles
   while Files.Count > NumFiles do
      Files.Delete(Files.Count-1);           // -1 since its 0 indexed

   UpdateMenu;
end;

procedure TMruMenu.OnMenuClick(Sender: TObject);
var
   i    : integer;
   Save : string;
begin
   // Make sure OnSelectFile is "Assigned"
   if not Assigned(FOnSelFile) then exit;

   i := (Sender as TMenuItem).Tag;
   // Call the OnSelectFile event with the correct File Name
   Save := Files[i];
   FOnSelFile(Self, Save);

   // Bring it to the top of the list
   AddFile(Save);
end;

procedure TMruMenu.UpdateMenu;
var
   ParMenu : TMenuItem;
   M       : TMenuItem;
   i       : integer;
   s       : string;
begin
   // Make sure the menu exists
   if FMainMenu=nil then exit;

   // Don't do anything at design time
   if csDesigning in ComponentState then exit;

   // Find our Parent Menu using its Group Number
   ParMenu := nil;
   for i := 0 to FMainMenu.Items.Count-1 do
      if FMainMenu.Items[i].GroupIndex = FParGroup then
         ParMenu := FMainMenu.Items[i];
   if ParMenu=nil then exit; // raise Exception.Create('No Parent Menu with given Group Number');

   // Delete the old Mru files from the menu (if they are there)
   for i := ParMenu.Count-1 downto 0 do      // Need to go backwards because we are Deleting
      if ParMenu.Items[i].GroupIndex = FMruGroup then begin
         M := ParMenu.Items[i];
         ParMenu.Delete(i);
         M.Free;
      end;

   if Files.Count=0 then exit;   // Don't want the seperator bar if there are no files

   // Add the seperator bar
   M              := TMenuItem.Create(Self);
   M.Caption      := '-';
   M.GroupIndex   := FMruGroup;
   ParMenu.Add(M);

   // Add the new files to the bottom of the menu
   for i := 0 to Files.Count-1 do begin
      if FShowPath then s := '&' + IntToStr(i+1) + ' ' + Files[i]
         else s := '&' + IntToStr(i+1) + ' ' + ExtractFileName(Files[i]);
      M            := TMenuItem.Create(Self);
      M.Caption    := s;
      M.GroupIndex := FMruGroup;
      M.OnClick    := OnMenuClick;
      M.Tag        := i;
      ParMenu.Add(M);
   end;
end;

procedure TMruMenu.Loaded;
begin
   ConnectToMenu;
end;

procedure TMruMenu.ConnectToMenu;
begin
   // Don't do anything at design time
   if csDesigning in ComponentState then exit;

   // Load in the Most Recently Used (Mru) files from the Mru file
   try
      Files.LoadFromFile(MruFilePath);
   except
      on Exception do begin end;       // Do nothing
   end;

   // Append them to the parent Menu
   UpdateMenu;
end;

function TMruMenu.MruFilePath: string;
begin
   if ExtractFilePath(FMruFile)='' then
      Result := ExtractFilePath(Application.ExeName) + FMruFile
   else
      Result := FMruFile;
end;

procedure TMruMenu.Clear;
begin
   Files.Clear;
   UpdateMenu;
end;

// This procedure is necessary to properly unhook from the MainMenu.
// It basically handles the message when the MainMenu is destroyed
// before the MruMenu.

procedure TMruMenu.Notification(AComponent: TComponent; Operation: TOperation);
begin
   inherited Notification(AComponent, Operation);

   if (AComponent = ParentMenu) and (Operation = opRemove)
      then ParentMenu := nil;
end;


end.
