unit TableUtils;

interface

uses SysUtils, Classes, DB, DBTables;

{
   Assumptions:

   - The first field that has "Required = True" is assumed to be the Primary Key
}

type
   TIECallback = procedure(RecordNum: integer; const Msg: string; DS: TDataSet) of object;
   TEachTableProc = procedure(Table: TTable) of object;

   // Iterating Tables
   procedure ForEachTableIn(Owner: TComponent; CallBack: TEachTableProc);

   // General String
   procedure FindReplace(var s: string; const Find, Rep: string);

   // Import / Export
   function  GetNumVisFields(DS: TDataSet): integer;
   function  GetVisibleField(DS: TDataSet; Index: integer): TField;
   procedure ExportTableToExcel(DS: TDataSet; SlowMode: boolean);
   procedure ExportTableToStringList(DS: TDataSet; SL: TStringList);
   procedure ImportTableFromStringList(DS: TDataSet; SL: TStringList; Callback: TIECallback);

   // Primary Key
   function  GetRequiredField(DataSet: TDataSet): TField;
   function  GetLastPK(Table: TTable): integer;


implementation

{$IFDEF VER100}
uses ComObj, Dialogs;
{$ELSE}
uses OleAuto, Dialogs;
{$ENDIF}

procedure FindReplace(var s: string; const Find, Rep: string);
var
   Done : boolean;
   p    : integer;
begin
   if Pos(Find, Rep)<>0 then raise Exception.Create('Find string is in Replace string!');
   repeat
      p := Pos(Find, s);
      Done := (p = 0);
      if not Done then begin
         s := Copy(s, 1, p-1) + Rep + Copy(s, p+Length(Find), Length(s));
      end;
   until Done;
end;


function GetNumVisFields(DS: TDataSet): integer;
var
   i : integer;
begin
   Result := 0;
   for i := 0 to DS.FieldCount-1 do begin
      if DS.Fields[i].Visible then Result := Result + 1;
   end;
end;


function GetVisibleField(DS: TDataSet; Index: integer): TField;
var
   i, j : integer;
begin
   i := 0;
   j := 0;
   Result := nil;
   while (i < DS.FieldCount) do begin
      if DS.Fields[i].Visible then begin
         if Index = j then Result := DS.Fields[i];
         j := j + 1;
      end;

      i := i + 1;
   end;
end;


procedure ExportTableToExcel(DS: TDataSet; SlowMode: boolean);
var
   XL       : Variant;
   Xarr     : Variant;
   Range1   : Variant;
   Range2   : Variant;
   Cell1    : Variant;
   Cell2    : Variant;
   xs, ys   : integer;
   i, j     : integer;
   s        : string;
begin
   try
      XL := GetActiveOLEObject('Excel.Application');
   except
      XL := CreateOLEObject('Excel.Application');
   end;

   XL.Visible := True;
   XL.Workbooks.Add;

   xs := GetNumVisFields(DS);
   ys := DS.RecordCount + 1;
   XArr := VarArrayCreate([1,ys,1,xs], varVariant);

   j := 1;
   for i := 0 to xs-1 do begin
      XArr[j, i+1] := GetVisibleField(DS, i).DisplayName;
   end;
   j := j + 1;

   DS.First;
   while not DS.EOF do begin
      for i := 0 to xs-1 do begin
         s := GetVisibleField(DS, i).AsString;
         FindReplace(s, #13, '');               // Excel converts them to boxes
         XArr[j, i+1] := s;
      end;
      j := j + 1;
      DS.Next;
   end;

   Range1 := XL.Cells;

   Cell1 := Range1.Item[1,1];
   Cell2 := Range1.Item[ys,xs];

   Range2 := Range1.Range[Cell1, Cell2];

   if SlowMode then begin
      for i := 1 to xs do begin
         for j := 1 to ys do begin
            Range1.Item[j,i].Value := XArr[j, i];
         end;
      end;
   end else begin
      Range2.Value := XArr;
   end;

   Range2.Columns.AutoFit;
   Range2.Rows.AutoFit;
   // Range2.AutoFormat;
   // XL.Quit;
end;


procedure ExportTableToStringList(DS: TDataSet; SL: TStringList);
var
   i  : integer;
   ts : TStringList;
   s  : string;
begin
   ts := TStringList.Create;
   SL.Clear;
   DS.First;

   // Add the Field Names
   ts.Clear;
   for i := 0 to DS.FieldCount-1 do begin
      if DS.Fields[i].FieldKind <> fkData then continue;
      ts.Add(DS.Fields[i].FieldName);
   end;
   SL.Add(ts.CommaText);

   // Add the Data
   while not DS.EOF do begin
      ts.Clear;
      for i := 0 to DS.FieldCount-1 do begin
         if DS.Fields[i].FieldKind <> fkData then continue;
         s := DS.Fields[i].AsString;
         FindReplace(s, #13#10, '\n');
         FindReplace(s, #9, '\t');
         ts.Add(s);
      end;
      SL.Add(ts.CommaText);

      DS.Next;
   end;
   ts.Free;
end;


procedure ImportTableFromStringList(DS: TDataSet; SL: TStringList; Callback: TIECallback);
var
   i, Line  : integer;
   ts, fs   : TStringList;
   NumErr   : integer;
   s        : string;
begin
   ts := TStringList.Create;
   fs := TStringList.Create;

   // Add the Field Names
   fs.CommaText := SL.Strings[0];

   // Add the Data
   NumErr := 0;
   for Line := 1 to SL.Count-1 do begin
      ts.CommaText := SL.Strings[Line];

      // if Assigned(Callback) then Callback(Line, 'Before Append', DS);
      DS.Append;
      // if Assigned(Callback) then Callback(Line, 'After Append', DS);
      for i := 0 to ts.Count-1 do begin
         s := ts.Strings[i];
         FindReplace(s, '\n', #13#10);
         FindReplace(s, '\t', #9);
         DS.FieldByName(fs.Strings[i]).AsString := s;
      end;
      try
         // if Assigned(Callback) then Callback(Line, 'Before Post', DS);
         DS.Post;
         if Assigned(Callback) then Callback(Line, 'After Post', DS);
      except
         on Exception do NumErr := NumErr + 1;
      end;
   end;
   ts.Free;
   fs.Free;

   if NumErr > 0 then begin
      ShowMessage(Format('Importing to Table %s had %d errors!', [DS.Name, NumErr]));
   end;
end;


procedure ForEachTableIn(Owner: TComponent; CallBack: TEachTableProc);
var
   i : integer;
   c : TComponent;
begin
   Assert(Assigned(CallBack));
   for i := 0 to Owner.ComponentCount-1 do begin
      c := Owner.Components[i];
      if (c is TTable) then CallBack(c as TTable);
   end;
end;


function GetRequiredField(DataSet: TDataSet): TField;
var
   i : integer;
begin
   Result := nil;
   for i := 0 to DataSet.FieldCount-1 do begin
      if DataSet.Fields[i].Required then begin
         Result := DataSet.Fields[i];
         break;
      end;
   end;
end;

function GetLastPK(Table: TTable): integer;
var
   PK       : TField;
   OldIdx   : string;
   bm       : TBookMark;
begin
   OldIdx := '';
   PK := GetRequiredField(Table);
   Assert(PK<>nil);

   bm := Table.GetBookmark;
   if Table.IndexFieldNames<>PK.FieldName then
      OldIdx := Table.IndexFieldNames;
   Table.IndexFieldNames := PK.FieldName;
   Table.Last;
   Result := PK.AsInteger;
   Table.GotoBookMark(bm);
   Table.FreeBookmark(bm);

   if OldIdx<>'' then Table.IndexFieldNames := OldIdx;
end;



end.
